function [B dB] = BB2dDeg2Pn(v,x,type)
%    [B dB]= BB2dDeg2Pmv(v,x,type)
% Evaluate generalized quadratic BB functions
% and gradients in a convex polygon
%
% Inputs:
% v    : [x1 y1; x2 y2; ...; xn yn], the n vertices of the polygon, ccw
% x    : [x(1) x(2)], the point at which the functions are evaluated
% type : type of barycentric coordinate:
%        type=1 => Wachspress
%        type=2 => MV local
%        type=3 => bilinear, if n=4.
%        type=4 => family.
% Outputs:
% B    : output basis functions = [B_11; B_12; B_22; ...; B_2n+1]
% dB   : output gradient of basis functions = [dB_11; dB_12; ...; dB_2n+1]
%
% Authors: Michael Floater and Ming-Jun Lai
% e-mail: michaelf@ifi.uio.no and mjlai@uga.edu
% Created: Mar 2014, using Matlab R2013b
% Modified: Nov 2014, gradients added
% Modified: March, 2015 for multiple points.
% Copyright 2015
% This is based on the results in the publication:
%  Polygonal spline spaces and  the numerical solution of the Poisson equation,
%    Michael Floater and Ming-Jun Lai
%  SIAM J. Numerical Analysis, accepted on  2015.
%
  m=size(x,1);
  n = size(v,1);
  B = zeros(2*n,m);
  dxB = zeros(2*n,m);dyB=dxB;
  phi=zeros(n,m); dxphi=phi; dyphi=phi;
 if type==1
%[phi dxphi dyphi] = wachspress2d(v,x);
[phi dxphi dyphi] = wachspress2dP(v,x);
 else
     error('type has to be 1')
 end
%  mod1 = inline('mod(i-1,n)+1','i','n');

  C = zeros(n,1);

  for i=1:n
    ip1 = mod1(i+1,n);
    im1 = mod1(i-1,n);
    C(i) = det([v(i,:)-v(im1,:); v(ip1,:)-v(i,:)]);
  end

  for i=1:n
    ip1 = mod1(i+1,n);
    ip2 = mod1(i+2,n);
    im1 = mod1(i-1,n);
    vi = zeros(3,2);
    vi(1,:) = v(im1,:);
    vi(2,:) = v(i,:);
    vi(3,:) = v(ip1,:);
%    [lami dlami] = baryTriang(vi,x);
   [lami dxlami,dylami] = baryTriangP(vi,x);
    vip1 = zeros(3,2);
    vip1(1,:) = v(i,:);
    vip1(2,:) = v(ip1,:);
    vip1(3,:) = v(ip2,:);
%    [lamip1 dlamip1] = baryTriang(vip1,x);
   [lamip1 dxlamip1,dylamip1] = baryTriangP(vip1,x);
    B(2*i-1,:) = phi(i,:) .* lami(2,:);
    dxB(2*i-1,:) = dxphi(i,:) .* lami(2,:) + phi(i,:) .* dxlami(2,:);
    dyB(2*i-1,:) = dyphi(i,:) .* lami(2,:) + phi(i,:) .* dylami(2,:);
    B(2*i,:) = phi(i,:) .* lami(3,:) + phi(ip1,:) .* lamip1(1,:);
    dxB(2*i,:) = dxphi(i,:) .* lami(3,:) + phi(i,:) .* dxlami(3,:) +... 
              dxphi(ip1,:) .* lamip1(1,:) + phi(ip1,:) .* dxlamip1(1,:);
    dyB(2*i,:) = dyphi(i,:) .* lami(3,:) + phi(i,:) .* dylami(3,:) +... 
              dyphi(ip1,:) .* lamip1(1,:) + phi(ip1,:) .* dylamip1(1,:);
  end
 dB=[dxB ;dyB];
