function [L dL] = Lag2dDeg2(v,x,type)
%
% Evaluate generalized quadratic Lagrange functions
% and gradients in a convex polygon
%
% Inputs:
% v    : [x1 y1; x2 y2; ...; xn yn], the n vertices of the polygon, ccw
% x    : [x(1) x(2)], the point at which the functions are evaluated
% type : type of barycentric coordinate:
%        type=1 => Wachspress
%        type=2 => MV local
%        type=3 => bilinear, if n=4.
%        type=4 => family.
% Outputs:
% L  : output basis functions = [L_11; L_12; L_22; ...; L_2n+1]
% dL : output gradient of basis functions = [dL_11; dL_12; ...; dL_2n+1]
%
% Author: Michael Floater
% e-mail: michaelf@ifi.uio.no
% Created: Mar 2014, using Matlab R2013b
% Modified: Nov 2014, gradients added
% Copyright 2014

  n = size(v,1);

  L = zeros(2*n,1);
  dL = zeros(2*n,2);

  [B dB] = BB2dDeg2L(v,x,type);

  mod1 = inline('mod(i-1,n)+1','i','n');

  for i=1:n
    im1 = mod1(i-1,n);
    L(2*i-1) = B(2*i-1) - 0.5 * (B(2*im1)+B(2*i));
    L(2*i)   = 2*B(2*i);
    dL(2*i-1,:) = dB(2*i-1,:) - 0.5 * (dB(2*im1,:)+dB(2*i,:));
    dL(2*i,:)   = 2*dB(2*i,:);
  end

