function [phi dphi] = baryTriangL(v,x)
%
% Evaluate the three barycentric coordinates
% and their gradients in a triangle.
%
% Inputs:
% v    : [x1 y1; x2 y2; x3 y3], the 3 vertices of the triangle, ccw
% x    : [x(1) x(2)], the point at which the basis functions are computed
% Outputs:
% phi  : output coordinates = [phi_1; phi_2; phi_3]
% dphi : output gradients of the coordinates = [dphi_1; dphi_2; dphi_3]
%
% Author: Michael Floater
% e-mail: michaelf@ifi.uio.no
% Created: Nov 2014, using Matlab R2014a
% Copyright 2014

phi  = zeros(3,1);
dphi = zeros(3,2);
e = zeros(3,2);
e(1,:) = v(2,:)-v(1,:);
e(2,:) = v(3,:)-v(2,:);
e(3,:) = v(1,:)-v(3,:);
%C = det([e(1,:); e(2,:)]);
C = e(1,1)*e(2,2) - e(2,1)*e(1,2);
d = zeros(3,2);
for i=1:3
  d(i,:) = v(i,:)-x;
end
%phi(1) = det([d(2,:); d(3,:)]) / C;
%phi(2) = det([d(3,:); d(1,:)]) / C;
%phi(3) = det([d(1,:); d(2,:)]) / C;
phi(1) = d(2,1)*d(3,2) - d(3,1)*d(2,2);
phi(2) = d(3,1)*d(1,2) - d(3,2)*d(1,1);
phi(3) = d(1,1)*d(2,2) - d(1,2)*d(2,1);
phi = phi / C;
if nargout == 2
  rot = @(y) [-y(2), y(1)];
  dphi(1,:) = rot(e(2,:)) / C;
  dphi(2,:) = rot(e(3,:)) / C;
  dphi(3,:) = rot(e(1,:)) / C;
end
